@@ -17,7 +17,7 @@ _ = gpodder.gettext
 __title__ = _('Enqueue/Resume in media players')
 __description__ = _('Add a context menu item for enqueueing/resuming playback of episodes in installed media players')
 __authors__ = 'Thomas Perl <thp@gpodder.org>, Bernd Schlapsi <brot@gmx.info>'
-__doc__ = 'http://wiki.gpodder.org/wiki/Extensions/EnqueueInMediaplayer'
+__doc__ = 'https://gpodder.github.io/docs/extensions/enqueueinmediaplayer.html'
 __payment__ = 'https://flattr.com/submit/auto?user_id=BerndSch&url=http://wiki.gpodder.org/wiki/Extensions/EnqueueInMediaplayer'
 __category__ = 'interface'
 __only_for__ = 'gtk'
@@ -212,6 +212,9 @@ PLAYERS = [
     # Clementine, http://www.clementine-player.org/
     FreeDesktopPlayer('clementine', 'Clementine', ['clementine', '--append']),
 
+    # Strawberry, https://www.strawberrymusicplayer.org/
+    FreeDesktopPlayer('strawberry', 'Strawberry', ['strawberry', '--append']),
+
     # Parole, http://docs.xfce.org/apps/parole/start
     FreeDesktopPlayer('parole', 'Parole', ['parole', '-a']),
 
